@@ -24,9 +24,9 @@ class ViewerTab(QWidget):
         self.debug = debug
         self.refreshRequired = True
         self.setWindowTitle('Viewer')
-        self.selected_mode = 4
         self.settings = {}
         self.UVW = deque()
+        self.settings['Selected mode'] = 4
         self.settings['Atom scaling'] = 0.5
         self.settings['Maximum displacement']  = 1.0
         self.settings['Bond colour']           = [  80,  80,  80, 255 ]
@@ -50,6 +50,7 @@ class ViewerTab(QWidget):
         self.cell_corners = None
         # element_colours is a dictionary
         self.element_colours = elemental_colours
+        self.settings['Element colours'] = None
         self.element_names = []
         self.species       = []
         self.element_coloured_buttons = []
@@ -69,11 +70,11 @@ class ViewerTab(QWidget):
         # Mode numbering starts at 1
         #
         self.selected_mode_sb = QSpinBox(self)
-        if self.frequencies_cm1 is not None:
+        if self.frequencies_cm1 is not None and len(self.frequencies_cm1) > 1:
             self.selected_mode_sb.setRange(1,len(self.frequencies_cm1))
         else:
             self.selected_mode_sb.setRange(1,2000)
-        self.selected_mode_sb.setValue(self.selected_mode)
+        self.selected_mode_sb.setValue(self.settings['Selected mode'])
         self.selected_mode_sb.setToolTip('Set the mode to be visualised')
         self.selected_mode_sb.valueChanged.connect(self.on_selected_mode_changed)
         label = QLabel('Select phonon mode', self)
@@ -346,6 +347,7 @@ class ViewerTab(QWidget):
         colour = colourChooser.currentColor()
         rgba = [ colour.red(), colour.green(), colour.blue(), colour.alpha() ]
         self.element_colours[text] = rgba
+        self.settings['Element colours'] = [ self.element_colours[el] for el in self.species ]
         self.refreshRequired = True
         self.refresh()
         return
@@ -422,16 +424,20 @@ class ViewerTab(QWidget):
 
     def on_selected_mode_changed(self):
         debugger.print('on_selected_mode_changed')
-        self.selected_mode = self.selected_mode_sb.value()
-        debugger.print('on selected_mode change ', self.selected_mode)
-        self.frequency_le.setText('{:.5f}'.format(self.notebook.settingsTab.frequencies_cm1[self.selected_mode-1]))
+        if self.frequencies_cm1 is None or len(self.frequencies_cm1) < 1:
+           debugger.print('on_selected_mode_changed aborting')
+           return
+        debugger.print('on selected_mode change mode was ', self.settings['Selected mode'])
+        self.settings['Selected mode'] = self.selected_mode_sb.value()
+        debugger.print('on selected_mode change mode is now ', self.settings['Selected mode'])
+        self.frequency_le.setText('{:.5f}'.format(self.frequencies_cm1[self.settings['Selected mode']-1]))
         self.opengl_widget.deleteArrows()
-        maxR = np.max( np.abs(np.array(self.UVW[self.selected_mode-1])))
+        maxR = np.max( np.abs(np.array(self.UVW[self.settings['Selected mode']-1])))
         if maxR < 1.0E-8:
             maxR = 1.0E-8
             self.plot_type_index = 2
         arrow_scaling = self.settings['Maximum displacement'] / maxR
-        for uvw in self.UVW[self.selected_mode-1]:
+        for uvw in self.UVW[self.settings['Selected mode']-1]:
             self.opengl_widget.addArrows( self.settings['Arrow colour'],self.settings['Arrow radius'], uvw, arrow_scaling )
         self.calculate()
         self.plot()
@@ -463,7 +469,6 @@ class ViewerTab(QWidget):
             return
         QApplication.setOverrideCursor(Qt.WaitCursor)
         # Assemble the settingsTab settings
-        settings = self.notebook.settingsTab.settings
         self.frequencies_cm1 = self.notebook.settingsTab.frequencies_cm1
         self.selected_mode_sb.setRange(1,len(self.frequencies_cm1))
         # Get the cell with whole molecules from the analysis tab
@@ -501,17 +506,17 @@ class ViewerTab(QWidget):
         # CalculatePhasePositions stores all the sphere and bond information
         self.calculatePhasePositions()
         # Add the arrows
-        maxR = np.max( np.abs(np.array(self.UVW[self.selected_mode-1])))
+        debugger.print('calculate: Selected mode',self.settings['Selected mode'])
+        maxR = np.max( np.abs(np.array(self.UVW[self.settings['Selected mode']-1])))
         if maxR < 1.0E-8:
             maxR = 1.0E-8
             self.plot_type_index = 2
         arrow_scaling = self.settings['Maximum displacement'] / maxR
         self.opengl_widget.deleteArrows()
-        for uvw in self.UVW[self.selected_mode-1]:
+        for uvw in self.UVW[self.settings['Selected mode']-1]:
             self.opengl_widget.addArrows( self.settings['Arrow colour'],self.settings['Arrow radius'], uvw, arrow_scaling )
         self.opengl_widget.setRotationCentre( centreOfMassXYZ )
         self.opengl_widget.setImageSize()
-        self.refreshRequired = False
         QApplication.restoreOverrideCursor()
         debugger.print('Finished:: calculate')
         return
@@ -534,7 +539,7 @@ class ViewerTab(QWidget):
         # we need the number of phase steps to be odd
         if self.settings['Number of phase steps']%2 == 0:
             self.settings['Number of phase steps'] += 1
-        UVW = np.array( self.UVW[self.selected_mode-1] )
+        UVW = np.array( self.UVW[self.settings['Selected mode']-1] )
         maxR = np.amax(np.abs(UVW))
         if maxR < 1.0E-8:
             maxR = 1.0E-8
@@ -579,7 +584,7 @@ class ViewerTab(QWidget):
         phases = np.arange(-1.0, 1.0+delta-1.0E-10, delta)
         with open(filename,'w') as fd:
             for phase_index,phase in enumerate(phases):
-                description = 'mode_'+str(self.selected_mode-1)+'_phase_'+str(phase)
+                description = 'mode_'+str(self.settings['Selected mode']-1)+'_phase_'+str(phase)
                 unitcell.set_xyz_coordinates(self.newXYZ[phase_index])
                 unitcell.write_cif(description,file_=fd)
 
@@ -637,11 +642,12 @@ class ViewerTab(QWidget):
         #
         for w in self.findChildren(QWidget):
             w.blockSignals(True)
-        self.selected_mode_sb.setValue(self.selected_mode)
-        try:
-            self.frequency_le.setText('{:.5f}'.format(self.notebook.settingsTab.frequencies_cm1[self.selected_mode-1]))
-        except:
-            self.frequency_le.setText('{:.5f}'.format(0.0))
+        self.frequencies_cm1 = self.notebook.settingsTab.frequencies_cm1
+        if self.frequencies_cm1 is not None and len(self.frequencies_cm1) > 1:
+            self.selected_mode_sb.setRange(1,len(self.frequencies_cm1))
+            self.selected_mode_sb.setValue(self.settings['Selected mode'])
+            self.frequency_le.setText('{:.5f}'.format(self.frequencies_cm1[self.settings['Selected mode']-1]))
+        debugger.print('refresh: selected mode is now',self.selected_mode_sb.value())
         self.atom_scaling_sb.setValue(self.settings['Atom scaling'])
         self.bond_radius_sb.setValue(self.settings['Bond radius'])
         self.cell_radius_sb.setValue(self.settings['Cell radius'])
@@ -667,6 +673,11 @@ class ViewerTab(QWidget):
             self.element_names = self.unit_cell.element_names
             self.species = self.reader.getSpecies()
             debugger.print('refresh - species ',self.species)
+            if self.settings['Element colours'] is not None:
+                debugger.print('refresh - processing colours ',self.species)
+                for el,colour in zip(self.species,self.settings['Element colours']):
+                    self.element_colours[el] = colour
+            self.settings['Element colours'] = [ self.element_colours[el] for el in self.species ]
         count = self.element_coloured_hbox.count()
         if count == 0:
             debugger.print('initialising element colours widget')
@@ -700,6 +711,7 @@ class ViewerTab(QWidget):
             w.blockSignals(False)
         self.calculate()
         self.plot()
+        self.refreshRequired = False
         QApplication.restoreOverrideCursor()
         debugger.print('Finished:: refresh')
         return
